diff --git a/contrib/dblink/dblink.c b/contrib/dblink/dblink.c
index fd7c8e6..7fd5909 100644
*** a/contrib/dblink/dblink.c
--- b/contrib/dblink/dblink.c
*************** static void dblink_res_error(const char 
*** 102,107 ****
--- 102,108 ----
  static char *get_connect_string(const char *servername);
  static char *escape_param_str(const char *from);
  static int	get_nondropped_natts(Oid relid);
+ static void truncate_identifier_copy(char *ident, const char *s);
  
  /* Global */
  static remoteConn *pconn = NULL;
*************** getConnectionByName(const char *name)
*** 2198,2205 ****
  	if (!remoteConnHash)
  		remoteConnHash = createConnHash();
  
! 	MemSet(key, 0, NAMEDATALEN);
! 	snprintf(key, NAMEDATALEN - 1, "%s", name);
  	hentry = (remoteConnHashEnt *) hash_search(remoteConnHash,
  											   key, HASH_FIND, NULL);
  
--- 2199,2205 ----
  	if (!remoteConnHash)
  		remoteConnHash = createConnHash();
  
! 	truncate_identifier_copy(key, name);
  	hentry = (remoteConnHashEnt *) hash_search(remoteConnHash,
  											   key, HASH_FIND, NULL);
  
*************** createNewConnection(const char *name, re
*** 2230,2237 ****
  	if (!remoteConnHash)
  		remoteConnHash = createConnHash();
  
! 	MemSet(key, 0, NAMEDATALEN);
! 	snprintf(key, NAMEDATALEN - 1, "%s", name);
  	hentry = (remoteConnHashEnt *) hash_search(remoteConnHash, key,
  											   HASH_ENTER, &found);
  
--- 2230,2236 ----
  	if (!remoteConnHash)
  		remoteConnHash = createConnHash();
  
! 	truncate_identifier_copy(key, name);
  	hentry = (remoteConnHashEnt *) hash_search(remoteConnHash, key,
  											   HASH_ENTER, &found);
  
*************** deleteConnection(const char *name)
*** 2254,2262 ****
  	if (!remoteConnHash)
  		remoteConnHash = createConnHash();
  
! 	MemSet(key, 0, NAMEDATALEN);
! 	snprintf(key, NAMEDATALEN - 1, "%s", name);
! 
  	hentry = (remoteConnHashEnt *) hash_search(remoteConnHash,
  											   key, HASH_REMOVE, &found);
  
--- 2253,2259 ----
  	if (!remoteConnHash)
  		remoteConnHash = createConnHash();
  
! 	truncate_identifier_copy(key, name);
  	hentry = (remoteConnHashEnt *) hash_search(remoteConnHash,
  											   key, HASH_REMOVE, &found);
  
*************** get_connect_string(const char *servernam
*** 2390,2399 ****
  	StringInfo	buf = makeStringInfo();
  	ForeignDataWrapper *fdw;
  	AclResult	aclresult;
  
  	/* first gather the server connstr options */
! 	if (strlen(servername) < NAMEDATALEN)
! 		foreign_server = GetForeignServerByName(servername, true);
  
  	if (foreign_server)
  	{
--- 2387,2397 ----
  	StringInfo	buf = makeStringInfo();
  	ForeignDataWrapper *fdw;
  	AclResult	aclresult;
+ 	char		srvname[NAMEDATALEN];
  
  	/* first gather the server connstr options */
! 	truncate_identifier_copy(srvname, servername);
! 	foreign_server = GetForeignServerByName(srvname, true);
  
  	if (foreign_server)
  	{
*************** get_nondropped_natts(Oid relid)
*** 2484,2486 ****
--- 2482,2496 ----
  	relation_close(rel, AccessShareLock);
  	return nondropped_natts;
  }
+ 
+ static void
+ truncate_identifier_copy(char *ident, const char *s)
+ {
+ 	int			len;
+ 
+ 	len = strlen(s);
+ 	len = pg_mbcliplen(s, len, NAMEDATALEN - 1);
+ 
+ 	memcpy(ident, s, len);
+ 	ident[len] = '\0';
+ }